@@ -33,7 +33,7 @@ from enum import Enum
 from typing import Dict, List, Set, Any, Tuple, Union, Optional, NamedTuple, Generator, NoReturn, Sequence
 
 import networkx as nx
-from nexinfosys.command_generators.parser_ast_evaluators import ast_evaluator
+from nexinfosys.command_generators.parser_ast_evaluators import ast_evaluator, ast_to_string
 
 from nexinfosys.command_field_definitions import orientations
 from nexinfosys.command_generators import Issue
@@ -49,7 +49,7 @@ from nexinfosys.models.musiasem_concepts import ProblemStatement, Parameter, Fac
 from nexinfosys.models.musiasem_concepts_helper import find_quantitative_observations
 from nexinfosys.solving.flow_graph_outputs import export_solver_data
 from nexinfosys.solving.graph import InterfaceNode, InterfaceNodeHierarchy, NodeFloatComputedDict, \
-    ProcessorsRelationWeights, NodeFloatDict, ResultDict, ResultKey, ConflictResolution, FloatComputedTuple, \
+    ProcessorsRelationWeights, ResultDict, ResultKey, ConflictResolution, FloatComputedTuple, \
     ComputationSource, Computed, evaluate_parameters_for_scenario, SolvingException, \
     evaluate_numeric_expression_with_parameters, AstType
 from nexinfosys.solving.graph.computation_graph import ComputationGraph
@@ -82,13 +82,13 @@ cached_sorts = {}
 # @cached(cache={}, key=lambda cache_key, list: cache_key)
 def sort_something(cache_key, list_to_sort):
     return list_to_sort
-    c = cached_sorts.get(cache_key, [])
-    if len(c) != len(list_to_sort):
-        c = sorted(list_to_sort)
-        cached_sorts[cache_key] = c
-    else:
-        print("Cache hit")
-    return c
+    # c = cached_sorts.get(cache_key, [])
+    # if len(c) != len(list_to_sort):
+    #     c = sorted(list_to_sort)
+    #     cached_sorts[cache_key] = c
+    # else:
+    #     print("Cache hit")
+    # return c
 
 
 class MissingValueResolutionPolicy(Enum):
@@ -106,8 +106,8 @@ class ConflictResolutionAlgorithm:
         self.computation_sources_priority_list = computation_sources_priority_list
         self.aggregation_conflict_policy = aggregation_conflict_policy
 
-    def resolve(self, value1: FloatComputedTuple, value2: FloatComputedTuple) -> Tuple[
-        FloatComputedTuple, FloatComputedTuple]:
+    def resolve(self, value1: FloatComputedTuple, value2: FloatComputedTuple) \
+            -> Tuple[FloatComputedTuple, FloatComputedTuple]:
         assert value1.computation_source != value2.computation_source, \
             f"The computation sources of both conflicting values cannot be the same: {value1.computation_source}"
 
@@ -122,8 +122,8 @@ class ConflictResolutionAlgorithm:
                 return value2, value1
 
         # One of the values has been computed by aggregation while the other is an observation
-        if ifnull(value1.computation_source, value2.computation_source) in (
-        ComputationSource.PartOfAggregation, ComputationSource.InterfaceTypeAggregation):
+        if ifnull(value1.computation_source, value2.computation_source) in (ComputationSource.PartOfAggregation,
+                                                                            ComputationSource.InterfaceTypeAggregation):
             if value1.computation_source is None:
                 # value2 is computed value, value1 is existing value
                 return self.aggregation_conflict_policy.resolve(value2, value1)
@@ -159,7 +159,8 @@ def get_computation_sources_priority_list(s: str) -> List[ComputationSource]:
 
     if len(sources) != len(ComputationSource):
         raise SolvingException(
-            f"The priority list of computation sources should have length {len(ComputationSource)} but has length: {len(sources)}")
+            f"The priority list of computation sources should have "
+            f"length {len(ComputationSource)} but has length: {len(sources)}")
 
     if len(sources) != len(set(sources)):
         raise SolvingException(f"The priority list of computation sources cannot have duplicated values: {sources}")
@@ -187,15 +188,15 @@ def get_evaluated_observations_by_time(prd: PartialRetrievalDictionary) -> TimeO
 
         Each evaluated observation is stored as a tuple:
         * First: the evaluated result as a float or the prepared AST
-        * Second: the observation
+        * Second: the observation.
 
-    :param prd: the global objects dictionary
+    :param prd: the dictionary of global objects
     :return: a time dictionary with a list of observation on each time
     """
     observations: TimeObservationsType = defaultdict(list)
     state = State()
 
-    # Get all observations by time
+    # Get all observations, grouped by time step (including generic time steps, like "Year" or "Month")
     for observation in find_quantitative_observations(prd, processor_instances_only=True):
         # Try to evaluate the observation value
         value, ast, _, issues = evaluate_numeric_expression_with_parameters(observation.value, state)
@@ -250,8 +251,8 @@ def check_type_consistency_from_all_time_periods(time_periods: List[str]) -> str
     return period_type
 
 
-def split_observations_by_relativeness(observations_by_time: TimeObservationsType) -> Tuple[
-    TimeObservationsType, TimeObservationsType]:
+def split_observations_by_relativeness(observations_by_time: TimeObservationsType) \
+        -> Tuple[TimeObservationsType, TimeObservationsType]:
     observations_by_time_norelative = defaultdict(list)
     observations_by_time_relative = defaultdict(list)
     for time, observations in observations_by_time.items():
@@ -264,53 +265,6 @@ def split_observations_by_relativeness(observations_by_time: TimeObservationsTyp
     return observations_by_time_norelative, observations_by_time_relative
 
 
-def compute_graph_results(comp_graph: ComputationGraph,
-                          existing_results: NodeFloatComputedDict,
-                          previous_known_nodes: Set[InterfaceNode],
-                          computation_source: ComputationSource) -> NodeFloatComputedDict:
-    # Filter results in graph
-    graph_params: NodeFloatDict = {k: v.value for k, v in existing_results.items() if k in comp_graph.nodes}
-
-    # Obtain nodes without a value
-    compute_nodes = comp_graph.nodes_not_in_container(graph_params)
-
-    if len(compute_nodes) == 0:
-        logging.debug("All nodes have a value. Nothing to solve.")
-        return {}
-
-    logging.debug(f"****** NODES: {comp_graph.nodes}")
-    logging.debug(f"****** UNKNOWN NODES: {compute_nodes}")
-
-    new_computed_nodes: Set[InterfaceNode] = {k for k in existing_results if k not in previous_known_nodes}
-    conflicts = comp_graph.compute_conflicts(new_computed_nodes, previous_known_nodes)
-
-    raise_error_if_conflicts(conflicts, graph_params, comp_graph.name)
-
-    results, _ = comp_graph.compute_values(compute_nodes, graph_params)
-
-    # Return only entries with a valid value and set the name
-    return_values: NodeFloatComputedDict = {}
-    for k, v in results.items():
-        if v is not None:
-            # v.name = k.name
-            return_values[k] = FloatComputedTuple(v, Computed.Yes, computation_source=computation_source)
-
-    return return_values
-
-
-def raise_error_if_conflicts(conflicts: Dict[InterfaceNode, Set[InterfaceNode]], graph_params: NodeFloatDict,
-                             graph_name: str):
-    conflict_strings: List[str] = []
-    for param, conf_params in conflicts.items():
-        if conf_params:
-            conf_params_string = "{" + ', '.join([f"{p} ({graph_params[p]})" for p in conf_params]) + "}"
-            conflict_strings.append(f"{param} ({graph_params[param]}) -> {conf_params_string}")
-
-    if conflict_strings:
-        raise SolvingException(
-            f"There are conflicts in the '{graph_name}' computation graph: {', '.join(conflict_strings)}")
-
-
 def create_interface_edges(edges: List[Tuple[Factor, Factor, Optional[str]]]) \
         -> Generator[Tuple[InterfaceNode, InterfaceNode, Dict], None, None]:
     for src, dst, weight in edges:
@@ -447,17 +401,19 @@ def replace_ast_variable_parts(ast: AstType, variable_conversion: Dict[str, str]
 def resolve_observations_with_only_values_and_parameters(parameters: CaseInsensitiveDict,
                                                          state: State, observations: ObservationListType,
                                                          observers_priority_list: Optional[List[str]], registry) \
-        -> Tuple[NodeFloatComputedDict, InterfaceNodeAstDict]:
+        -> Tuple[NodeFloatComputedDict, InterfaceNodeAstDict, List[str]]:
     resolved_observations: NodeFloatComputedDict = {}
     unresolved_observations_with_interfaces: InterfaceNodeAstDict = {}
 
     p_set = set([k.lower() for k in parameters.keys()])
+    exceptions = []
     for expression, obs in observations:
         interface_params: Dict[str, str] = {}
         obs_new_value: Optional[str] = None
-        value, ast, params, issues = evaluate_numeric_expression_with_parameters(expression, state)
+        value, ast, not_found_vars, issues = evaluate_numeric_expression_with_parameters(expression, state)
         # If it is an expression (not a literal value), if there is an interface with same name as a parameter, assume
         # the interface, but if there is not one, take the parameter value
+
         if isinstance(expression, dict):
             # Parameter?  Interface?  Interface with Value?  BEHAVIOR
             # ----------  ----------  ---------------------  --------
@@ -472,27 +428,32 @@ def resolve_observations_with_only_values_and_parameters(parameters: CaseInsensi
 
             _, p = ast_evaluator(expression, State(), None, issues)
             p = set([p_.lower() for p_ in p]).intersection(p_set)
-            if len(p) > 0:
-                params = set()
+            if len(p) > 0:  # There is at least one parameter in the expression
+                params_overridden_by_interface = set()
+                # Get the parameter names from the expression and see if there is an interface with the same name
                 for p_ in p:
                     interfaces: Sequence[Factor] = registry.get(
                         Factor.partial_key(processor=obs.factor.processor, name=p_))
                     if len(interfaces) == 1:
                         # print(f"Assuming interface '{p_}' for observation '{expression}'")
-                        params.add(p_)
+                        params_overridden_by_interface.add(p_)
                         value = None
                         ast = expression
+                not_found_vars.update(params_overridden_by_interface)
 
         if value is None:
-            interface_params, params, issues = convert_params_to_extended_interface_names(params, obs, registry)
+            interface_params, not_found_vars, issues = convert_params_to_extended_interface_names(not_found_vars, obs, registry)
             if interface_params and not issues:
                 ast = replace_ast_variable_parts(ast, interface_params)
                 obs_new_value = replace_string_from_dictionary(obs.value, interface_params)
             else:
-                raise SolvingException(
-                    f"Cannot evaluate expression '{expression}' for observation at interface '{obs.factor.name}'. "
-                    f"Params: {params}. Issues: {', '.join(issues)}"
-                )
+                interface_params = not_found_vars
+                exceptions.append(f"Cannot evaluate expression '{ast_to_string(expression)}' for observation at interface '{obs.factor.name}', processor '{obs.factor.processor.full_hierarchy_name}'. "
+                                  f"Params: {not_found_vars}. Issues: {', '.join(issues)}")
+                # raise SolvingException(
+                #     f"Cannot evaluate expression '{expression}' for observation at interface '{obs.factor.name}'. "
+                #     f"Params: {not_found_vars}. Issues: {', '.join(issues)}"
+                # )
 
         # Get observer name
         observer_name = obs.observer.name if obs.observer else None
@@ -535,7 +496,7 @@ def resolve_observations_with_only_values_and_parameters(parameters: CaseInsensi
                                                              Computed.No, observer_name)
             unresolved_observations_with_interfaces.pop(node, None)
 
-    return resolved_observations, unresolved_observations_with_interfaces
+    return resolved_observations, unresolved_observations_with_interfaces, exceptions
 
 
 def resolve_observations_with_interfaces(
@@ -796,7 +757,8 @@ def get_processor_partof_relations(glb_idx: PartialRetrievalDictionary) \
 
 
 def compute_hierarchy_graph_results(
-        graph: ComputationGraph, params: NodeFloatComputedDict,
+        graph: ComputationGraph,
+        computed_values: NodeFloatComputedDict,
         prev_computed_values: NodeFloatComputedDict,
         conflict_resolution_algorithm: ConflictResolutionAlgorithm,
         computation_source: ComputationSource) \
@@ -805,7 +767,7 @@ def compute_hierarchy_graph_results(
     Compute nodes in a graph hierarchy and also mark conflicts with existing values (params)
 
     :param graph: hierarchy as a graph of interface nodes
-    :param params: all nodes with a known value
+    :param computed_values: all nodes with a known value
     :param prev_computed_values: all nodes that have been previously computed with same computation source
     :param conflict_resolution_algorithm: algorithm for resolution of conflicts
     :param computation_source: source of computation
@@ -817,7 +779,7 @@ def compute_hierarchy_graph_results(
         input_values: List[FloatExp.ValueWeightPair] = []
 
         for n, weight in inputs:
-            res_backward = compute_node(n)
+            res_backward: Optional[FloatExp] = compute_node(n)
 
             # If node 'n' is a 'split' only one result is needed to compute the result
             if split:
@@ -851,7 +813,7 @@ def compute_hierarchy_graph_results(
             sum_children = solve_inputs(sort_something((node, "r"), graph.reverse_inputs(node)),
                                         graph.get_direct_node_split(node))
 
-        float_value = params.get(node)
+        float_value = computed_values.get(node)
         if sum_children is not None:
             # New value has been computed
             sum_children.name = node.name
@@ -864,6 +826,16 @@ def compute_hierarchy_graph_results(
 
                 new_values[node] = taken_conflicts[node]
                 return_value = taken_conflicts[node].value
+                # if new_computed_value.computation_source != float_value.computation_source:
+                #     # Conflict here: applies strategy
+                #     taken_conflicts[node], dismissed_conflicts[node] = \
+                #         conflict_resolution_algorithm.resolve(new_computed_value, float_value)
+                #
+                #     new_values[node] = taken_conflicts[node]
+                #     return_value = taken_conflicts[node].value
+                # else:
+                #     return_value = float_value.value
+                #     print(f"WARNING: same source? HG. {node.name}")
             else:
                 new_values[node] = new_computed_value
                 return_value = new_computed_value.value
@@ -890,7 +862,8 @@ def compute_hierarchy_graph_results(
 
 
 def compute_hierarchy_aggregate_results(
-        tree: InterfaceNodeHierarchy, params: NodeFloatComputedDict,
+        tree: InterfaceNodeHierarchy,
+        computed_values: NodeFloatComputedDict,
         prev_computed_values: NodeFloatComputedDict,
         conflict_resolution_algorithm: ConflictResolutionAlgorithm,
         missing_values_policy: MissingValueResolutionPolicy,
@@ -901,7 +874,7 @@ def compute_hierarchy_aggregate_results(
     Compute aggregations of nodes in a hierarchy and also mark conflicts with existing values (params)
 
     :param tree: dictionary representing a hierarchy as a tree of interface nodes in the form [parent, set(child)]
-    :param params: all nodes with a known value
+    :param computed_values: all nodes with a known value
     :param prev_computed_values: all nodes that have been previously computed by aggregation
     :param conflict_resolution_algorithm: algorithm for resolution of conflicts
     :param missing_values_policy: policy for missing values when aggregating children
@@ -938,7 +911,7 @@ def compute_hierarchy_aggregate_results(
         if not invalidate_sum_children:
             sum_children = FloatExp.compute_weighted_addition(children_values)
 
-        float_value = params.get(node)
+        float_value = computed_values.get(node)
         if sum_children is not None:
             # New value has been computed
             sum_children.name = node.name
@@ -951,6 +924,16 @@ def compute_hierarchy_aggregate_results(
 
                 new_values[node] = taken_conflicts[node]
                 return_value = taken_conflicts[node].value
+                # if new_computed_value.computation_source != float_value.computation_source:
+                #     # Conflict here: applies strategy
+                #     taken_conflicts[node], dismissed_conflicts[node] = \
+                #         conflict_resolution_algorithm.resolve(new_computed_value, float_value)
+                #
+                #     new_values[node] = taken_conflicts[node]
+                #     return_value = taken_conflicts[node].value
+                # else:
+                #     return_value = float_value.value
+                #     print(f"WARNING: same source? AR. {node.name}")
             else:
                 new_values[node] = new_computed_value
                 return_value = new_computed_value.value
@@ -981,7 +964,7 @@ def init_processor_full_names(registry: PartialRetrievalDictionary):
 def flow_graph_solver(global_parameters: List[Parameter], problem_statement: ProblemStatement,
                       global_state: State, dynamic_scenario: bool) -> List[Issue]:
     """
-    A solver using the graph composed by the interfaces and the relationships (flows, part-of, scale, change-of-scale and relative-to)
+    A solver using the graph composed by the interfaces and the relationships (flows, part-of, scale, change-of-scale and relative-to).
 
     :param global_parameters: Parameters including the default value (if defined)
     :param problem_statement: ProblemStatement object, with scenarios (parameters changing the default)
@@ -996,33 +979,38 @@ def flow_graph_solver(global_parameters: List[Parameter], problem_statement: Pro
         glb_idx, _, _, _, _ = get_case_study_registry_objects(global_state)
         init_processor_full_names(glb_idx)
 
-        # Get available observations
+        # Get all the observations, parsed and evaluated, split into Absolute and Relative, then grouped-by Time Step
         time_absolute_observations, time_relative_observations = \
             split_observations_by_relativeness(get_evaluated_observations_by_time(glb_idx))
 
         if len(time_absolute_observations) == 0:
             return [Issue(IType.WARNING, f"No absolute observations have been found. The solver has nothing to solve.")]
 
-        # Get available interfaces
+        # Get all the interfaces of the model, and construct a set of InterfaceNodes
         interface_nodes: Set[InterfaceNode] = {InterfaceNode(i) for i in glb_idx.get(Factor.partial_key())}
 
         # Get hierarchies of processors and update interfaces to compute
-        partof_hierarchies, partof_weights = compute_partof_hierarchies(glb_idx, interface_nodes)
+        _ = compute_partof_hierarchies(glb_idx, interface_nodes)
+        partof_hierarchies: InterfaceNodeHierarchy = _[0]
+        partof_weights: ProcessorsRelationWeights = _[1]
 
         # Get hierarchies of interface types and update interfaces to compute
-        interfacetype_hierarchies = compute_interfacetype_hierarchies(glb_idx, interface_nodes)
+        interfacetype_hierarchies: InterfaceNodeHierarchy = compute_interfacetype_hierarchies(glb_idx, interface_nodes)
 
-        relations_flow, relations_scale, relations_scale_change = \
-            compute_flow_and_scale_relation_graphs(glb_idx, interface_nodes)
+        _ = compute_flow_and_scale_relation_graphs(glb_idx, interface_nodes)
+        relations_flow: nx.DiGraph = _[0]
+        relations_scale: nx.DiGraph = _[1]
+        relations_scale_change: nx.DiGraph = _[2]
 
         total_results: ResultDict = {}
 
+        # SCENARIOS - Outermost loop of the solver
         for scenario_name, scenario_params in problem_statement.scenarios.items():  # type: str, Dict[str, Any]
             logging.debug(f"********************* SCENARIO: {scenario_name}")
-            params = evaluate_parameters_for_scenario(global_parameters, scenario_params)
+            params: CaseInsensitiveDict[str, Any] = evaluate_parameters_for_scenario(global_parameters, scenario_params)
             scenario_state = State(params)
-            scenario_partof_weights = resolve_partof_weight_expressions(partof_weights, scenario_state,
-                                                                        raise_error=True)
+            scenario_partof_weights: ProcessorsRelationWeights = \
+                resolve_partof_weight_expressions(partof_weights, scenario_state, raise_error=True)
 
             # Get scenario parameters
             observers_priority_list = parse_string_as_simple_ident_list(
@@ -1038,6 +1026,7 @@ def flow_graph_solver(global_parameters: List[Parameter], problem_statement: Pro
             if missing_value_policy == MissingValueResolutionPolicy.UseZero:
                 missing_value_policies.append(MissingValueResolutionPolicy.UseZero)
 
+            # TIME STEPS - Second loop
             for time_period, absolute_observations in time_absolute_observations.items():
                 logging.debug(f"********************* TIME PERIOD: {time_period}")
 
@@ -1045,17 +1034,21 @@ def flow_graph_solver(global_parameters: List[Parameter], problem_statement: Pro
                 total_dismissed_results: NodeFloatComputedDict = {}
 
                 try:
-                    comp_graph_flow, comp_graph_scale, comp_graph_scale_change = \
-                        compute_flow_and_scale_computation_graphs(scenario_state,
+                    _ = compute_flow_and_scale_computation_graphs(scenario_state,
                                                                   time_relative_observations[time_period],
                                                                   relations_flow,
                                                                   relations_scale,
                                                                   relations_scale_change)
+                    comp_graph_flow: ComputationGraph = _[0]
+                    comp_graph_scale: ComputationGraph = _[1]
+                    comp_graph_scale_change: ComputationGraph = _[2]
 
-                    # Get final results from the absolute observations
-                    results, unresolved_observations_with_interfaces = \
-                        resolve_observations_with_only_values_and_parameters(params, scenario_state, absolute_observations,
-                                                                             observers_priority_list, glb_idx)
+                    _ = resolve_observations_with_only_values_and_parameters(
+                            params, scenario_state, absolute_observations, observers_priority_list, glb_idx)
+                    results: NodeFloatComputedDict = _[0]
+                    unresolved_observations_with_interfaces: InterfaceNodeAstDict = _[1]
+                    f_issues = _[2]
+                    issues.extend([Issue(IType.ERROR, f) for f in f_issues])
 
                     # Initializations
                     iteration_number = 1
@@ -1069,28 +1062,43 @@ def flow_graph_solver(global_parameters: List[Parameter], problem_statement: Pro
                                        scenario_partof_weights)
                     ]
 
-                    # START ITERATIVE SOLVING
+                    # ITERATIVE SOLVING, for a pair (Scenario, Time step)
 
                     # We first iterate with policy MissingValueResolutionPolicy.Invalidate trying to get as many results
-                    # we can without supposing zero for missing values.
+                    # we can without assuming zero for missing values.
                     # Second, if specified in paramater "NISSolverMissingValueResolutionPolicy" we try to get further
                     # results with policy MissingValueResolutionPolicy.UseZero.
                     for missing_value_policy in missing_value_policies:
                         previous_len_results = len(results) - 1
 
-                        # Iterate while the number of results is increasing
+                        # Iterate while there are new results
                         while len(results) > previous_len_results:
                             logging.debug(f"********************* Solving iteration: {iteration_number}")
                             previous_len_results = len(results)
 
                             for pi in processing_items:
+
+                                # Core of the solver
                                 if pi.source.is_aggregation():
-                                    new_results, taken_results, dismissed_results = compute_hierarchy_aggregate_results(
-                                        pi.hierarchy, results, pi.results, conflict_resolution_algorithm,
-                                        missing_value_policy, pi.source, pi.partof_weights)
+                                    _ = compute_hierarchy_aggregate_results(
+                                        pi.hierarchy,
+                                        results,
+                                        pi.results,
+                                        conflict_resolution_algorithm,
+                                        missing_value_policy,
+                                        pi.source,
+                                        pi.partof_weights)
                                 else:
-                                    new_results, taken_results, dismissed_results = compute_hierarchy_graph_results(
-                                        pi.hierarchy, results, pi.results, conflict_resolution_algorithm, pi.source)
+                                    _ = compute_hierarchy_graph_results(
+                                        pi.hierarchy,
+                                        results,
+                                        pi.results,
+                                        conflict_resolution_algorithm,
+                                        pi.source)
+
+                                new_results: NodeFloatComputedDict = _[0]
+                                taken_results: NodeFloatComputedDict = _[1]
+                                dismissed_results: NodeFloatComputedDict = _[2]
 
                                 pi.results.update(new_results)
                                 results.update(new_results)
@@ -1098,10 +1106,10 @@ def flow_graph_solver(global_parameters: List[Parameter], problem_statement: Pro
                                 total_dismissed_results.update(dismissed_results)
 
                             if unresolved_observations_with_interfaces:
-                                new_results, unresolved_observations_with_interfaces = \
-                                    resolve_observations_with_interfaces(
-                                        scenario_state, unresolved_observations_with_interfaces, results
-                                    )
+                                _ = resolve_observations_with_interfaces(
+                                        scenario_state, unresolved_observations_with_interfaces, results)
+                                new_results: NodeFloatComputedDict = _[0]
+                                unresolved_observations_with_interfaces: NodeFloatComputedDict = _[1]
                                 results.update(new_results)
 
                             iteration_number += 1
@@ -1130,6 +1138,7 @@ def flow_graph_solver(global_parameters: List[Parameter], problem_statement: Pro
                         HierarchicalNodeStructure.from_flow_computation_graph(comp_graph_flow, True),
                         HierarchicalNodeStructure.from_partof_aggregation(partof_hierarchies, scenario_partof_weights),
                         HierarchicalNodeStructure.from_interfacetype_aggregation(interfacetype_hierarchies)]
+
                     additional_hierarchical_structure = HierarchicalNodeStructure.from_flow_computation_graph(
                         comp_graph_flow, False)
 
@@ -1144,6 +1153,8 @@ def flow_graph_solver(global_parameters: List[Parameter], problem_statement: Pro
 
                 except SolvingException as e:
                     return [Issue(IType.ERROR, f"Scenario '{scenario_name}' - period '{time_period}'. {e.args[0]}")]
+            # End: Time step loop
+        # End: Scenario loop
 
         # Prepare all output information (store it in the global state, variable "datasets")
         #  - Indicators are calculated inside
@@ -1311,23 +1322,7 @@ def check_unresolved_nodes_in_computation_graphs(computation_graphs: List[Comput
     return issues
 
 
-def check_unresolved_nodes_in_aggregation_hierarchies(hierarchies: List[InterfaceNodeHierarchy],
-                                                      resolved_nodes: NodeFloatComputedDict) -> List[Issue]:
-    issues: List[Issue] = []
-    unresolved_nodes: Set[InterfaceNode] = set()
-
-    for hierarchy in hierarchies:
-        unresolved_nodes.update({n for n in hierarchy if n not in resolved_nodes})
-        for parent, children in hierarchy.items():
-            unresolved_nodes.update({n for n in children if n not in resolved_nodes})
-
-    if unresolved_nodes:
-        issues.append(Issue(IType.WARNING, f"The following nodes in aggregation hierarchies could not be "
-                                           f"evaluated: {unresolved_nodes}"))
-    return issues
-
-
-def compute_flow_and_scale_relation_graphs(registry, interface_nodes: Set[InterfaceNode]):
+def compute_flow_and_scale_relation_graphs(registry, interface_nodes: Set[InterfaceNode]) -> Tuple[nx.DiGraph, nx.DiGraph, nx.DiGraph]:
     # Compute Interfaces -Flow- relations (time independent)
     relations_flow = nx.DiGraph(
         incoming_graph_data=create_interface_edges(
